/**
 * @(#)Bits.java
 * @author JLMJ
 * @date 2001-10-04
 */
package minoids1;

public final class Bits
{
    public static final int MAX_BITS = 64;
    public static final long LOWER_BIT = 1L;
    public static final long UPPER_BIT = 1L << 63;
    
    private long[] data;
    private int numberOfBits;
    
    public Bits() {
        // empty
    }
    
    public Bits(long data) {
        numberOfBits = MAX_BITS;
        this.data = new long[1];
        this.data[0] = data;
    }
    
    public Bits(final Bits bits) {
        numberOfBits = bits.numberOfBits;
        data = new long[bits.data.length];
        for (int i=0; i < data.length; i++)
            data[i] = bits.data[i];
    }
    
    public Bits(int numberOfBits, int numberOfOnes) {
        set(numberOfBits, numberOfOnes);
    }
    
    public void set(int numberOfBits, int numberOfOnes) {
        if (numberOfOnes > numberOfBits)
            numberOfOnes = numberOfBits;
        this.numberOfBits = numberOfBits;
        int dataSize = numberOfBits / MAX_BITS +
            ((numberOfBits % MAX_BITS == 0) ? 0 : 1);
        data = new long[dataSize];
        for (int i=0; i < data.length; i++)
            data[0] = 0;
        for (int i=0; i < numberOfOnes; i++)
            shiftLeft(true);
    }
    
    public int getNumberOfBits() {
        return numberOfBits;
    }
    
    public void setBit(int bitPosition) {
        if (numberOfBits > bitPosition) {
            int total = MAX_BITS * data.length;
            data[bitPosition / total] |= 1L << (bitPosition % total);
        }
    }
    
    public boolean getBit(int bitPosition) {
        if (numberOfBits > bitPosition) {
            int total = MAX_BITS * data.length;
            long r = data[bitPosition / total] & 1L << (bitPosition % total);
            return r != 0;
        }
        return false;
    }
    
    public boolean isClearAfterAndWith(final Bits bits) {
        for (int i=0; i < data.length; i++)
            if ((data[i] & bits.data[i]) != 0)
                return false;
        return true;
    }
    
    public boolean isNotClearAfterAndWith(final Bits bits) {
        for (int i=0; i < data.length; i++)
            if ((data[i] & bits.data[i]) != 0)
                return true;
        return false;
    }
    
    public boolean isClear() {
        for (int i=0; i < data.length; i++)
            if (data[i] != 0)
                return false;
        return true;
    }
    
    public boolean isNotClear() {
        for (int i=0; i < data.length; i++)
            if (data[i] != 0)
                return true;
        return false;
    }
    
    public void doAndWith(final Bits bits) {
        for (int i=0; i < data.length; i++)
            data[i] &= bits.data[i];
    }
    
    public void doOrWith(final Bits bits) {
        for (int i=0; i < data.length; i++)
            data[i] |= bits.data[i];
    }
    
    public void doNot() {
        for (int i=0; i < data.length; i++) {
            data[i] = ~data[i];
        }
    }
    
    public boolean equals(final Bits bits) {
        for (int i=0; i < data.length; i++)
            if (data[i] != bits.data[i])
                return false;
        return true;
    }
    
    private boolean prev, next;
    
    public boolean shiftLeft(boolean insertedBit) {
        prev = insertedBit;
        for (int i=0; i < data.length; i++) {
            next = ((data[i] & UPPER_BIT) != 0);
            data[i] <<= 1;
            if (prev)
                data[i] |= LOWER_BIT;
            prev = next;
        }
        return next;
    }
    
    public boolean shiftRight(boolean insertBit) {
        prev = insertBit;
        for (int i=data.length - 1; i >= 0; i--) {
            next = ((data[i] & LOWER_BIT) != 0);
            data[i] >>>= 1;
            if (prev)
                data[i] |= UPPER_BIT;
            prev = next;
        }
        return next;
    }
    
    private char DOT = '.';
    private char ZERO = '0';
    private char ONE = '1';
    private char SPACE = ' ';
    
    public String toString() {
        // Upper data long can have less bits than MAX_BITS due size
        // can be lower the length of data.lenght * MAX_BITS
        int diff = data.length * MAX_BITS - numberOfBits;
        
        // The number of characters for the upper data long.
        int numberOfChars = (MAX_BITS - diff) + 1;
        // The number of characters for the rest of the data plus
        // separator between them to improve their reading.
        numberOfChars += (data.length - 1) * (MAX_BITS + 1);
        char[] out = new char[numberOfChars];
        int charPos = 0;
        
        // Fill up the first block
        long number = data[data.length - 1];
        long mask = UPPER_BIT >>> diff;
        boolean noOneYet = true;
        char c;
        for (int a = diff; a < MAX_BITS; a++) {
            if ((number & mask) == 0) {
                c = (noOneYet) ? DOT : ZERO;
            } else {
                noOneYet = false;
                c = ONE;
            }
            out[charPos++] = c;
            number <<= 1;
        }
        out[charPos++] = SPACE;
        
        // Fill up the rest of the blocks
        for (int a = data.length - 2; a >= 0; a--) {
            number = data[a];
            for (int b=0; b < MAX_BITS; b++) {
                if ((number & UPPER_BIT) == 0) {
                    c = (noOneYet) ? DOT : ZERO;
                } else {
                    noOneYet = false;
                    c = ONE;
                }
                out[charPos++] = c;
                number <<= 1;
            }
            out[charPos++] = SPACE;
        }
        out[charPos - 1] = '\0';

        return new String(out);
    }
    
    public long toLong() {
        return data[0];
    }
}
